cs = require "coffee-script"
fs = require "fs"
_ = require "underscore"

dk.plugin "model", ["transport", "assets"], ->
  allModels = {}
  storage = null
  modelsDir = process.cwd() + "/models"

  class BaseModel
    constructor: (@attributes={}) ->
      this.loadAttributes()
      this.dumpAttributes()

    dumpAttributes: ->
      _this = this
      atts = @attributes
      for property, type of @klass.properties
        if type == Date && _this[property] && _this[property].getTime
          atts[property] = _this[property].getTime()
        else
          atts[property] = _this[property]

    loadAttributes: (atts) ->
      _this = this
      atts ?= @attributes
      for property, val of atts
        type = @klass.properties[property]
        if type == Date
          _this[property] = new Date(val)
        else
          _this[property] = val

    save: (cb) ->
      this.dumpAttributes()
      @klass.save @attributes, cb

    destroy: (cb) ->
      this.dumpAttributes()
      @klass.destroy @attributes.id, cb

  class OnlyNeededToGenerateExtendMethods extends BaseModel

  classMethods = {
    storage: storage,

    properties: {}

    property: (name, type) ->
      this.properties[name] = type

    get: (id, cb) ->
      model = this
      storage.get id, (err, data) ->
        if err
          cb err
        else
          console.log "Got", data
          item = new model(data)
          cb null, item

    save: (atts, cb) ->
      model = this
      atts.dkModelName = this.modelName
      atts.updatedAt = Date.now()
      atts.createdAt = Date.now() unless atts.id
      console.log "Saving", atts
      storage.save atts, (err, data) ->
        if err
          cb err
        else
          atts.id = data.id
          item = new model(atts)
          cb null, item

    create: (atts, cb) ->
      this.save atts, cb

    destroy: (id, cb) ->
      console.log "Request to destroy ", id
      storage.destroy id, cb

    wrap: (atts) ->
      delete atts.dkModelName
      new this(atts)
      
  }

  serveModelFile = (name, suffix) ->
    dk.assets.package "/js/models/#{name}.js", ->
      code = if suffix == "js"
        fs.readFileSync "#{modelsDir}/#{name}.js"
      else if suffix == "coffee"
        cs.compile fs.readFileSync("#{modelsDir}/#{name}.coffee").toString(), bare: true
      "$(function() { #{code} });"

  loadAllModels = ->
    for filename in fs.readdirSync(modelsDir)
      if m = filename.match /^(.*)\.(js|coffee)/
        require "#{modelsDir}/#{filename}"
        serveModelFile m[1], m[2]

  loadModel = (name) ->
    unless allModels[name]
      formattedName = name.toLowerCase()
      reqStr = "#{modelsDir}/#{formattedName}"
      require reqStr
      serveModel formattedName

  modelBuilder = (name, cb) ->
    if cb
      dk.transport.on "model.#{name}.get", (query, cb) ->
        NewModel.get query.id, (err, obj) ->
          if err
            console.log "Error fetching #{name} #{query.id}: #{err}"
            cb null
          else
            cb obj.attributes

      dk.transport.on "model.#{name}.save", (atts, cb) ->
        NewModel.save atts, (err, obj) ->
          if err
            cb null
          else
            console.log "Just saved", obj.attributes
            cb obj.attributes

      dk.transport.on "model.#{name}.destroy", (atts, cb) ->
        NewModel.destroy atts.id, (err, success) ->
          if err
            cb false
          else
            cb true

      NewModel = eval """
        (function() {
          __extends(#{name}, BaseModel);
          function #{name}(atts) {
            this.klass = #{name};
            #{name}.__super__.constructor.call(this, atts);
          }
          return #{name};
        })();
      """

      _.extend NewModel, classMethods
      NewModel.modelName = name

      if modelBuilder.beforeInit
        modelBuilder.beforeInit.call NewModel

      cb NewModel
      allModels[name] = NewModel

      NewModel.property "id", String
      NewModel.property "createdAt", Date
      NewModel.property "updatedAt", Date

      if modelBuilder.afterInit
        modelBuilder.afterInit.call NewModel

      NewModel
    else
      # loadModel name
      allModels[name]

  modelBuilder.allModels = allModels

  modelBuilder.config = (opts) ->
    storage = opts.storage
    modelBuilder.storage = opts.storage
    if opts.dir
      modelDir = opts.dir

  modelBuilder.run = ->
    dk.assets.package "/js/dk-model.js", "#{__dirname}/../public/js/dk-model.coffee"
    loadAllModels()

  return modelBuilder

