dk.plugin "model", ["transport"], (name) ->
  allModels = {}

  class BaseModel
    constructor: (@attributes) ->
      this.loadAttributes()
      this.dumpAttributes()

    dumpAttributes: ->
      _this = this
      for property, type of @klass.properties
        if type == Date && _this[property] && _this[property].getTime
          @attributes[property] = _this[property].getTime()
        else
          @attributes[property] = _this[property]

    loadAttributes: (atts) ->
      _this = this
      atts ?= @attributes
      for property, val of atts
        type = @klass.properties[property]
        if type == Date
          _this[property] = new Date(val)
        else
          _this[property] = val

    save: (cb) ->
      this.dumpAttributes()
      @klass.save @attributes, cb

    destroy: (cb) ->
      this.dumpAttributes()
      @klass.destroy @attributes.id, cb

  class OnlyNeededToGenerateExtendMethods extends BaseModel

  classMethods = {
    get: (id, cb) ->
      model = this
      dk.transport.send "model.#{model.modelName}.get", {id: id}, (atts) ->
        if atts
          cb null, new model(atts)
        else
          cb "No Data", null

    save: (atts, cb) ->
      cb ?= ->
      model = this
      console.log "Saving atts", atts
      dk.transport.send "model.#{model.modelName}.save", atts, (newAtts) ->
        if newAtts
          cb null, new model(newAtts)
        else
          cb "Error saving record", null

    create: (atts, cb) ->
      this.save atts, cb

    destroy: (id, cb) ->
      model = this
      dk.transport.send "model.#{model.modelName}.destroy", id: id, (success) ->
        if success
          cb null, true
        else
          cb "Error destroying item", false

    properties: {}

    property: (name, type) ->
      this.properties[name] = type
  }

  modelBuilder = (name, cb) ->
    if cb
      NewModel = eval """
        (function() {
          __extends(#{name}, BaseModel);
          function #{name}(atts) {
            this.klass = #{name};
            #{name}.__super__.constructor.call(this, atts);
          }
          return #{name};
        })();
      """
      $.extend NewModel, classMethods
      NewModel.modelName = name

      if modelBuilder.beforeInit
        modelBuilder.beforeInit.call NewModel

      cb NewModel
      allModels[name] = NewModel

      NewModel.property "id", String
      NewModel.property "createdAt", Date
      NewModel.property "updatedAt", Date

      if modelBuilder.afterInit
        modelBuilder.afterInit.call NewModel

      NewModel
    else
      allModels[name]

  return modelBuilder
